@@ -1,5 +1,10 @@
-#![allow(clippy::upper_case_acronyms)] // Because breaking changes are not allowed
-use std::{fmt::Display, str::FromStr};
+#![allow(
+    clippy::upper_case_acronyms,
+    clippy::manual_non_exhaustive,
+    clippy::match_like_matches_macro
+)] // Kill clippy for MSRV
+use alloc::{borrow::ToOwned, string::String};
+use core::{fmt::Display, str::FromStr};
 
 ///
 /// The result of FromStr::from_str, when parsing a field (other than vendor),
@@ -9,8 +14,7 @@ pub struct UnknownError;
 
 ///
 /// The Architecture field of a target tuple
-#[non_exhaustive]
-#[derive(Copy, Clone, PartialEq, Eq, Debug)]
+#[derive(Copy, Clone, PartialEq, Eq, Hash, Debug)]
 pub enum Architecture {
     Unknown,
     I86,
@@ -46,6 +50,8 @@ pub enum Architecture {
     M6502,
     M65C02,
     SPC700,
+    #[doc(hidden)]
+    __Nonexhaustive,
 }
 
 impl FromStr for Architecture {
@@ -98,7 +104,7 @@ impl FromStr for Architecture {
 }
 
 impl Display for Architecture {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         self.canonical_name().fmt(f)
     }
 }
@@ -108,14 +114,6 @@ impl Architecture {
     /// This is equivalent to [`Self::from_str`], but returns [`Architecture::Unknown`], instead of an error,
     ///  on an unknown architecture.
     /// This is useful (in conjunction with an actual target name)
-    /// ## Example
-    /// ```
-    ///     use target_tuples::Architecture;
-    ///     let arch = Architecture::parse("i386");
-    ///     assert_eq!(arch,Architecture::I386);
-    ///     let arch2: Architecture = "i486".parse().unwrap();
-    ///     assert_ne!(arch,arch2);
-    /// ```
     pub fn parse(st: &str) -> Self {
         Self::from_str(st).unwrap_or(Architecture::Unknown)
     }
@@ -124,12 +122,6 @@ impl Architecture {
     /// Returns the canonical name of the target
     /// The canonical name, when passed into `[`Self::parse`] will yield an equivalent value,
     /// Formatting an Architecture yields this string
-    /// ## Examples
-    /// ```
-    ///#   use target_tuples::Architecture;
-    ///    let arch = Architecture::I386;
-    ///    assert_eq!(Architecture::parse(arch.canonical_name()),arch);
-    /// ```
     pub fn canonical_name(&self) -> &'static str {
         match self {
             Architecture::Unknown => "unknown",
@@ -166,10 +158,10 @@ impl Architecture {
             Architecture::M6502 => "6502",
             Architecture::M65C02 => "6502",
             Architecture::SPC700 => "spc700",
+            Architecture::__Nonexhaustive => unreachable!(),
         }
     }
 
-    #[allow(clippy::match_like_matches_macro)] // Cannot use matches! with MSRV 1.40, so just allow it
     pub fn is_x86(&self) -> bool {
         match self {
             Architecture::I86
@@ -188,7 +180,7 @@ impl Architecture {
 ///
 /// The Vendor field of a target tuple
 ///
-#[derive(Copy, Clone, PartialEq, Eq, Debug)]
+#[derive(Copy, Clone, PartialEq, Eq, Hash, Debug)]
 pub enum Vendor {
     Unknown,
     Apple,
@@ -206,10 +198,12 @@ pub enum Vendor {
     SUSE,
     OpenEmbedded,
     WDC,
+    #[doc(hidden)]
+    __Nonexhaustive,
 }
 
 impl FromStr for Vendor {
-    type Err = std::convert::Infallible;
+    type Err = core::convert::Infallible;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         Ok(match s {
@@ -236,7 +230,7 @@ impl FromStr for Vendor {
 }
 
 impl Display for Vendor {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         self.canonical_name().fmt(f)
     }
 }
@@ -245,15 +239,6 @@ impl Vendor {
     /// Parses the Vendor in a "lossy" manner
     /// This is equivalent to [`Self::from_str`].
     /// Note that an unknown vendor is not considered an error.
-    ///
-    /// ## Example
-    /// ```
-    ///     use target_tuples::Vendor;
-    ///     let vendor = Vendor::parse("pc");
-    ///     assert_eq!(vendor,Vendor::PC);
-    ///     let vendor2: Vendor = "pc".parse().unwrap();
-    ///     assert_eq!(vendor,vendor2);
-    /// ```
     pub fn parse(s: &str) -> Self {
         Self::from_str(s).unwrap()
     }
@@ -262,12 +247,6 @@ impl Vendor {
     /// Returns the canonical name of the vendor
     /// The canonical name, when passed into `[`Self::parse`] will yield an equivalent value,
     /// Formatting a Vendor yields this string
-    /// ## Examples
-    /// ```
-    ///    use target_tuples::Vendor;
-    ///    let vendor = Vendor::Apple;
-    ///    assert_eq!(Vendor::parse(vendor.canonical_name()),vendor);
-    /// ```
     pub fn canonical_name(&self) -> &'static str {
         match self {
             Vendor::Apple => "apple",
@@ -286,14 +265,14 @@ impl Vendor {
             Vendor::SUSE => "suse",
             Vendor::OpenEmbedded => "oe",
             Vendor::WDC => "wdc",
+            Vendor::__Nonexhaustive => unreachable!(),
         }
     }
 }
 
 ///
 /// The Operating System Field of a target tuple
-#[derive(Copy, Clone, PartialEq, Eq, Debug)]
-#[non_exhaustive]
+#[derive(Copy, Clone, PartialEq, Eq, Hash, Debug)]
 pub enum OS {
     Unknown,
 
@@ -335,6 +314,8 @@ pub enum OS {
     PhantomOS,
     SNES, // Not an OS, but the currently config.sub places it in the os field
     NES,  // likewise
+    #[doc(hidden)]
+    __Nonexhaustive,
 }
 
 impl FromStr for OS {
@@ -387,7 +368,7 @@ impl FromStr for OS {
 }
 
 impl Display for OS {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         self.canonical_name().fmt(f)
     }
 }
@@ -395,15 +376,6 @@ impl Display for OS {
 impl OS {
     /// Parses the OS in a "lossy" manner
     /// This is equivalent to [`Self::from_str`], except that [`OS::Unknown`] is returned, instead of an error, on an unknown OS Field
-    ///
-    /// ## Example
-    /// ```
-    ///     use target_tuples::OS;
-    ///     let os = OS::parse("linux");
-    ///     assert_eq!(os,OS::Linux);
-    ///     let os2: OS = "linux".parse().unwrap();
-    ///     assert_eq!(os,os2);
-    /// ```
     pub fn parse(s: &str) -> Self {
         Self::from_str(s).unwrap_or(Self::Unknown)
     }
@@ -412,12 +384,6 @@ impl OS {
     /// Returns the canonical name of the operating system
     /// The canonical name, when passed into `[`Self::parse`] will yield an equivalent value,
     /// Formatting an OS yields this string
-    /// ## Examples
-    /// ```
-    ///    use target_tuples::OS;
-    ///    let os = OS::PS4;
-    ///    assert_eq!(OS::parse(os.canonical_name()),os);
-    /// ```
     pub fn canonical_name(&self) -> &'static str {
         match self {
             OS::Unknown => "unknown",
@@ -459,14 +425,14 @@ impl OS {
             OS::PhantomOS => "phantom",
             OS::SNES => "snes",
             OS::NES => "nes",
+            OS::__Nonexhaustive => unreachable!(),
         }
     }
 }
 
 ///
 /// The Environment field of target tuples
-#[derive(Copy, Clone, PartialEq, Eq, Debug)]
-#[non_exhaustive]
+#[derive(Copy, Clone, PartialEq, Eq, Hash, Debug)]
 pub enum Environment {
     Unknown,
     GNU,
@@ -492,6 +458,9 @@ pub enum Environment {
 
     PhantomStandard,
     PhantomKernel,
+
+    #[doc(hidden)]
+    __Nonexhaustive,
 }
 
 impl FromStr for Environment {
@@ -518,7 +487,7 @@ impl FromStr for Environment {
             x if x.starts_with("coreclr") => Self::CoreCLR,
             x if x.starts_with("simulator") => Self::Simulator,
             x if x.starts_with("macabi") => Self::MacABI,
-            x if x.starts_with("pstd") || x.starts_with("user") => Self::PhantomStandard,
+            x if x.starts_with("pcore") || x.starts_with("user") => Self::PhantomStandard,
             x if x.starts_with("pkrnl") || x.starts_with("kernel") => Self::PhantomKernel,
             _ => return Err(UnknownError),
         })
@@ -526,7 +495,7 @@ impl FromStr for Environment {
 }
 
 impl Display for Environment {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         self.canonical_name().fmt(f)
     }
 }
@@ -534,15 +503,6 @@ impl Display for Environment {
 impl Environment {
     /// Parses the Environment name in a "lossy" manner
     /// This is equivalent to [`Self::from_str`], except that [`Environment::Unknown`] is returned, instead of an error, on an unknown OS Field
-    ///
-    /// ## Example
-    /// ```
-    ///     use target_tuples::Environment;
-    ///     let env = Environment::parse("gnu");
-    ///     assert_eq!(env,Environment::GNU);
-    ///     let env2: Environment = "gnu".parse().unwrap();
-    ///     assert_eq!(env,env2);
-    /// ```
     pub fn parse(s: &str) -> Self {
         Self::from_str(s).unwrap_or(Self::Unknown)
     }
@@ -551,12 +511,6 @@ impl Environment {
     /// Returns the canonical name of the environment
     /// The canonical name, when passed into [`Self::parse`] will yield an equivalent value,
     /// Formatting an Environment yields this string
-    /// ## Examples
-    /// ```
-    ///    use target_tuples::Environment;
-    ///    let os = Environment::Musl;
-    ///    assert_eq!(Environment::parse(os.canonical_name()),os);
-    /// ```
     pub fn canonical_name(&self) -> &'static str {
         match self {
             Environment::Unknown => "unknown",
@@ -581,14 +535,14 @@ impl Environment {
             Environment::MacABI => "macabi",
             Environment::PhantomStandard => "user",
             Environment::PhantomKernel => "kernel",
+            Environment::__Nonexhaustive => unreachable!(),
         }
     }
 }
 
 ///
 /// The object format used by a target
-#[derive(Copy, Clone, PartialEq, Eq, Debug)]
-#[non_exhaustive]
+#[derive(Copy, Clone, PartialEq, Eq, Hash, Debug)]
 pub enum ObjectFormat {
     Unknown,
     XCoff,
@@ -601,6 +555,9 @@ pub enum ObjectFormat {
     Xo65,
     O65,
     WlaObj,
+
+    #[doc(hidden)]
+    __Nonexhaustive,
 }
 
 impl FromStr for ObjectFormat {
@@ -624,7 +581,7 @@ impl FromStr for ObjectFormat {
 }
 
 impl Display for ObjectFormat {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         self.canonical_name().fmt(f)
     }
 }
@@ -632,15 +589,6 @@ impl Display for ObjectFormat {
 impl ObjectFormat {
     /// Parses the ObjectFormat name in a "lossy" manner, from the end of the Environment field
     /// This is equivalent to [`Self::from_str`], except that [`ObjectFormat::Unknown`] is returned, instead of an error, on an unknown OS Field
-    ///
-    /// ## Example
-    /// ```
-    ///     use target_tuples::ObjectFormat;
-    ///     let of = ObjectFormat::parse("gnuelf");
-    ///     assert_eq!(of,ObjectFormat::Elf);
-    ///     let of2: ObjectFormat = "pstdelf".parse().unwrap();
-    ///     assert_eq!(of,of2);
-    /// ```
     pub fn parse(s: &str) -> Self {
         Self::from_str(s).unwrap_or(Self::Unknown)
     }
@@ -649,14 +597,6 @@ impl ObjectFormat {
     /// Returns the canonical name of the object format
     /// The canonical name, when passed into [`Self::parse`] will yield an equivalent value,
     /// Formatting an ObjectFormat yields this string
-    /// ## Examples
-    /// ```
-    ///    use target_tuples::ObjectFormat;
-    ///    let os = ObjectFormat::MachO;
-    ///    assert_eq!(ObjectFormat::parse(os.canonical_name()),os);
-    /// ```
-    ///
-
     pub fn canonical_name(&self) -> &'static str {
         match self {
             ObjectFormat::Unknown => "unknown",
@@ -669,6 +609,7 @@ impl ObjectFormat {
             ObjectFormat::Xo65 => "xo65",
             ObjectFormat::O65 => "o65",
             ObjectFormat::WlaObj => "wlaobj",
+            ObjectFormat::__Nonexhaustive => unreachable!(),
         }
     }
 }
@@ -689,7 +630,7 @@ impl ObjectFormat {
 ///
 #[derive(Clone, Debug)]
 pub struct Target {
-    full: std::string::String,
+    full: String,
     arch: Architecture,
     vendor: Option<Vendor>,
     // Invariant:
@@ -795,7 +736,7 @@ impl FromStr for Target {
 }
 
 impl Display for Target {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         self.arch.fmt(f)?;
         f.write_str("-")?;
         if let Some(vendor) = &self.vendor {
@@ -823,6 +764,28 @@ impl Display for Target {
     }
 }
 
+impl core::hash::Hash for Target {
+    fn hash<H: core::hash::Hasher>(&self, state: &mut H) {
+        self.arch.hash(state);
+        self.vendor.hash(state);
+        self.os.hash(state);
+        self.env.hash(state);
+        self.objfmt.hash(state);
+    }
+}
+
+impl PartialEq for Target {
+    fn eq(&self, other: &Self) -> bool {
+        self.arch == other.arch
+            && self.vendor == other.vendor
+            && self.os == other.os
+            && self.env == other.env
+            && self.objfmt == other.objfmt
+    }
+}
+
+impl Eq for Target {}
+
 impl Target {
     ///
     /// Gets the exact name of the target tuple.
@@ -944,15 +907,15 @@ impl Target {
     }
 
     ///
-    /// Gets the value of the `os` field, or unknown if the os was omitted
-    pub fn get_operating_system(&self) -> OS {
-        self.os.unwrap_or(OS::Unknown)
+    /// Gets the value of the `os` field
+    pub fn get_operating_system(&self) -> Option<OS> {
+        self.os
     }
 
     ///
     /// Gets the value of the `env` field, or unknown if the environment was omitted
-    pub fn get_environment(&self) -> Environment {
-        self.env.unwrap_or(Environment::Unknown)
+    pub fn get_environment(&self) -> Option<Environment> {
+        self.env
     }
 
     ///
@@ -972,7 +935,7 @@ impl Target {
             env,
             objfmt,
         };
-        ret.full = format!("{}", &ret);
+        ret.full = alloc::format!("{}", &ret);
         ret
     }
 
